<?php

 // THIS FILE IS PART OF WEBMATIC3 (Webmatic cube)
 // www.valarsoft.com

 defined('WM3_cEXEC') or die('Denied');

 class wm3GUIList extends wm3GUI
  {
   // Arrays
   private $_WIDTHS;
   private $_CAPTION_LABELS,$_CAPTION_WIDTHS,$_CAPTION_COLSPAN;
   private $_COLSPAN,$_COLORS;
   private $_SEPARATOR,$_BAR_VERTICAL,$_BAR_HORIZONTAL;
   private $_EXTRA_HEADER;

   // Variables
   private $columns,$page,$max_page_items,$total_items;
   private $render_table,$render_pages_navigator,$render_caption;
   private $page_navigator_field,$page_navigator_id,$page_navigator_rs,$use_auto_page;
   private $extra_header;
   private $use_colors,$invert_colors;
   private $objects_per_row;
   private $bottom_content;
   private $search_form;
   private $currentID;
   private $pages_navigator_link;
   private $empty_label;

   function __construct($w3,$data,$title = '',$colspan = array(),$colors = array())
    {
     // $data must be an array of objects
	 parent::__construct($w3,$data,$title);
	 if(isset($this->_DATA[0])) $this->columns = count((array)$this->_DATA[0]);
	 else                       $this->columns = 0;
	 $this->_COLSPAN               = $colspan;
	 $this->_COLORS                = $colors;
     $this->_SEPARATOR             = array();
     $this->_BAR_VERTICAL          = array();
     $this->_BAR_HORIZONTAL        = array();
	 $this->page                   = 0;
	 $this->max_page_items         = 12;
	 $this->objects_per_row        = 1;
	 $this->render_table           = true;
	 $this->render_pages_navigator = true;
	 $this->render_caption         = true;
	 $this->use_colors             = true;
     $this->page_navigator_field   = '';
     $this->page_navigator_id      = 0;
     $this->page_navigator_rs      = 0;
     $this->use_auto_page          = false;
	 $this->invert_colors          = false;
	 $this->pages_navigator_link   = '';
	 $this->search_form            = '';
	 $this->bottom_content         = '';
	 $this->_EXTRA_HEADER          = array();
	 $this->_CAPTION_LABELS        = array();
	 $this->_CAPTION_WIDTHS        = array();
	 $this->_CAPTION_COLSPAN       = array();
	 $this->_WIDTHS                = array();
	 $this->currentID              = -1;
	 $this->empty_label            = $w3->getWord(1217);
    }

   public function render()
    {
	 // Adjust _DATA objects array if needed
     if($this->objects_per_row > 1) $this->collapseDataArray();

	 // Render list header
	 $str = $this->renderHeader();

     // Render list body
     $str .= $this->renderBody();

     // Render list bottom
     $str .= $this->renderBottom();

	 return $str;
	}

   public function renderHeader()
    {
     // Render a generic header for a list
     $str = '';

     if(!$this->title)
	  {
	   if($this->render_table)
	    {
         ob_start();
         require($this->_skin->getListHeader(false));
         $str = ob_get_contents();
         ob_end_clean();
		 return $str;
	    }
	   else return '';
      }

     if($this->render_table)
	  {
       ob_start();
       require($this->_skin->getListHeader(true));
       $str .= ob_get_contents();
       ob_end_clean();
	  }
     ob_start();
	 $colspan = "colspan=\"".$this->columns."\"";
	 $content = $this->title;
     require($this->_skin->getListHeaderTitle());
     $str .= ob_get_contents();
     ob_end_clean();

	 // Skin for header list search
     if($this->search_form)
	  {
       ob_start();
	   $content = $this->search_form;
       require($this->_skin->getListHeaderSearch());
       $str .= ob_get_contents();
       ob_end_clean();
	  }

     foreach($this->_EXTRA_HEADER as $value)
      {
       ob_start();
	   $colspan = "colspan=\"".$this->columns."\"";
	   $content = $value;
       require($this->_skin->getListHeaderTitle());
       $str .= ob_get_contents();
       ob_end_clean();
      }

     return $str;
    }

   public function renderBody()
    {
     // Render the list body (caption and items)
     $str = '';

	 // Render list caption
	 if($this->render_caption) $str .= $this->makeCaption();

     // Check for empty list (exit with a message if there are 0 items)
	 if(count($this->_DATA) == 0) { $str .= $this->makeEmptyItems(); return $str; }

     $min = 0; $max = count($this->_DATA);

     // If $this->use_auto_page is true
     // If count($_DATA) is greater than the max page items to show ($this->max_page_items)
     // (as occur when data are taked from database with limit sql statament) we must show only
     // the items under the current page ($this->page) so we must calculate $min and $max
     if($this->render_pages_navigator && $this->use_auto_page)
      {
       if(count($this->_DATA) > $this->max_page_items)
        {
         $min = $this->page * $this->max_page_items;
         $max = min($this->page * $this->max_page_items + $this->max_page_items,count($this->_DATA));
        }
       if($min >= $max) // Invalid page (page out of range) return no items
        {
		 $this->_DATA = array(); $str .= $this->makeEmptyItems(); return $str;
		}
      }

     // Render list content
     for($i = $min; $i < $max; $i++)
      {
       if(isset($this->_SEPARATOR[$i]))      $separator      = $this->_SEPARATOR[$i];      else $separator      = false;
       if(isset($this->_BAR_VERTICAL[$i]))   $bar_vertical   = $this->_BAR_VERTICAL[$i];   else $bar_vertical   = false;
       if(isset($this->_BAR_HORIZONTAL[$i])) $bar_horizontal = $this->_BAR_HORIZONTAL[$i]; else $bar_horizontal = false;
       if(isset($this->_COLSPAN[$i]))
        {
         if($this->use_colors) {
                                if(isset($this->_COLORS[$i])) $str .= $this->makeItem($this->_DATA[$i],$this->_COLORS[$i],$this->currentID == $i,$this->_COLSPAN[$i],$separator,$bar_vertical,$bar_horizontal);
                                else                          $str .= $this->makeItem($this->_DATA[$i],$i,$this->currentID == $i,$this->_COLSPAN[$i],$separator,$bar_vertical,$bar_horizontal);
	                           }
         else $str .= $this->makeItem($this->_DATA[$i],0,$this->currentID == $i,$this->_COLSPAN[$i],$separator,$bar_vertical,$bar_horizontal);
        }
       else
        {
		 if($this->use_colors) {
		                        if(isset($this->_COLORS[$i])) $str .= $this->makeItem($this->_DATA[$i],$this->_COLORS[$i],$this->currentID == $i,array(),$separator,$bar_vertical,$bar_horizontal);
		                        else                          $str .= $this->makeItem($this->_DATA[$i],$i,$this->currentID == $i,array(),$separator,$bar_vertical,$bar_horizontal);
	                           }
	     else $str .= $this->makeItem($this->_DATA[$i],0,$this->currentID == $i,array(),$separator,$bar_vertical,$bar_horizontal);
	    }
      }

     return $str;
    }

   public function renderBottom()
    {
     // Render the list bottom
     if(count($this->_DATA) == 0) return ''; // Do not render bottom if there are no body items

     $str = '';
     if($this->render_pages_navigator && $this->total_items > $this->max_page_items)
      $str .= $this->makeBottom(wm3GUIList::makePagesNavigator($this->_w3,$this->total_items,$this->max_page_items,$this->page,$this->pages_navigator_link,$this->page_navigator_rs,$this->page_navigator_field,$this->page_navigator_id).$this->bottom_content);
     else
      $str .= $this->makeBottom($this->bottom_content);

     return $str;
    }

   public function makeCaption()
    {
     // Create a generic caption for a list
	 ob_start();
     require($this->_skin->getListCaptionStart());
     $str = ob_get_contents();
	 ob_end_clean();
	 $max = $this->columns;
     for($i = 0; $i < $max; $i++)
      {
	   if(isset($this->_CAPTION_COLSPAN[$i])) { $colspan = "colspan=\"".$this->_CAPTION_COLSPAN[$i]."\""; $max -= $this->_CAPTION_COLSPAN[$i] - 1; } else $colspan = '';
       if(isset($this->_CAPTION_LABELS[$i])) $content = $this->_CAPTION_LABELS[$i]; else $content = '';
	   if(!$content) $content = '&nbsp;';
       if(isset($this->_CAPTION_WIDTHS[$i])) $width = "width=\"".$this->_CAPTION_WIDTHS[$i]."%\""; else $width = '';
	   ob_start();
	   require($this->_skin->getListCaption());
	   $str .= ob_get_contents();
       ob_end_clean();
      }
	 ob_start();
     require($this->_skin->getListCaptionEnd());
     $str .= ob_get_contents();
     ob_end_clean();

     return $str;
    }

   public function makeEmptyItems()
    {
     // Create a generic item for an empty list
	 ob_start();
     require($this->_skin->getListItemStart(0,false,false,false,false));
     $str = ob_get_contents();
	 ob_end_clean();

     ob_start();
     $colspan = "colspan=\"".$this->columns."\"";
	 $content = $this->empty_label;
	 $width   = '';
	 require($this->_skin->getListItem(0,false,false,false,false));
	 $str .= ob_get_contents();
     ob_end_clean();

	 ob_start();
     require($this->_skin->getListItemEnd(0,false,false,false));
     $str .= ob_get_contents();
     ob_end_clean();

	 if(!$this->render_table) return $str;
     ob_start();
	 require($this->_skin->getListBottomEnd());
	 $str .= ob_get_contents();
     ob_end_clean();

     return $str;
    }

   public function makeItem($row,$p,$highlighted,$_COLSPAN = array(),$separator = false, $bar_vertical = false, $bar_horizontal = false)
    {
     // Create a generic item line for a list
     if($this->invert_colors) $p++; // Invert list colors
	 ob_start();
     require($this->_skin->getListItemStart($p,$highlighted,$separator,$bar_vertical,$bar_horizontal));
     $str = ob_get_contents();
	 ob_end_clean();
	 $i = 0;
	 foreach($row as $value)
      {
       $colspan = '';
       if(isset($_COLSPAN[$i]))
        if($_COLSPAN[$i] > 1) $colspan = "colspan=\"$_COLSPAN[$i]\"";
       $content = $value;
       if(isset($this->_WIDTHS[$i])) $width = "width=\"".$this->_WIDTHS[$i]."%\""; else $width = '';
       ob_start();
	   require($this->_skin->getListItem($p,$highlighted,$separator,$bar_vertical,$bar_horizontal));
	   $str .= ob_get_contents();
       ob_end_clean();
       $i++;
      }
	 ob_start();
     require($this->_skin->getListItemEnd($p,$highlighted,$bar_vertical,$bar_horizontal));
     $str .= ob_get_contents();
     ob_end_clean();
     return $str;
    }

   public function makeBottom($content = '')
    {
     // Create a generic bottom for a list
     if(!$this->render_table) return '';
     if(!$content)
      {
       ob_start();
	   require($this->_skin->getListBottomEnd());
       $str = ob_get_contents();
       ob_end_clean();
       return $str;
      }
     ob_start();
	 $colspan = "colspan=\"".$this->columns."\"";
     require($this->_skin->getListBottom());
     $str = ob_get_contents();
     ob_end_clean();
     ob_start();
     require($this->_skin->getListBottomEnd());
	 $str .= ob_get_contents();
     ob_end_clean();
     return $str;
    }

   public function collapseDataArray()
    {
	 // If needed collapse $objects_per_row objects in a single row and overwrite
	 // the new $_DATA array with collapsed objects
	 $k = -1;
     for($i = 0; $i < count($this->_DATA); $i++)
      {
       if($i % $this->objects_per_row == 0) $k++;
       foreach($this->_DATA[$i] as $key => $value)
        {
	     $field = $key.($i%$this->objects_per_row);
         $_AUX[$k]->$field = $value;
        }
      }
     if(isset($_AUX))
      {
       $m = count($this->_WIDTHS) - count((array)$_AUX[$k]); // Missing fields to complete a row
       for($i = 0; $i < $m; $i++) { $field = "dummy".$i; $_AUX[$k]->$field = ''; }
       $this->_DATA = $_AUX;
       // Update $columns value with the new _DATA array
       if(isset($this->_DATA[0])) $this->columns = count((array)$this->_DATA[0]);
	   else                       $this->columns = 0;
      }
    }

   public static function fixNavigatorPage($w3,$currentID,$total_items,$max_page_items,$page,$field,$rs)
    {
     // Set the right navigator page value
     if($total_items % $max_page_items == 0 && $page == (int)($total_items / $max_page_items)) $page--;
	 if($currentID)
	  {
	   $position = $w3->getDBObj()->getRSPosition($rs,$field,$currentID);
	   $page = (int)($position / $max_page_items);
      }
     return $page;
    }

   public static function makePagesNavigator($w3,$total,$max,$pg,$link,$rs = 0,$field = '',$_ID = 0)
    {
     // Make right links to create a pages navigator (eventually using a result set)
     $icon_first = "<img src=\"".WM3_cPATH_DATA."misc_first.png\" width=\"14\" height=\"14\" align=\"absmiddle\" border=\"0\">";
     $icon_last  = "<img src=\"".WM3_cPATH_DATA."misc_last.png\"  width=\"14\" height=\"14\" align=\"absmiddle\" border=\"0\">";
     $icon_prev  = "<img src=\"".WM3_cPATH_DATA."misc_prev.png\"  width=\"14\" height=\"14\" align=\"absmiddle\" border=\"0\">";
     $icon_next  = "<img src=\"".WM3_cPATH_DATA."misc_next.png\"  width=\"14\" height=\"14\" align=\"absmiddle\" border=\"0\">";
     if($rs)
      {
	   $i = 0; $str = '';
	   $prev_fl = 1; $next_fl = 0;
       while($row = $w3->getDBObj()->fetch($rs,false))
        {
	     if($i == 0) { $firstID = $row[$field]; $prevID = $row[$field]; }
	     $lastID = $row[$field];
	     if($next_fl) { $nextID = $row[$field]; $next_fl = 0; }
         if($row[$field] == $_ID) { $str .= ($i+1)." "; $next_fl = 1; $prev_fl = 0; $nextID = $row[$field]; }
         else                     { $str .= "<a href=\"$link&$field=$row[$field]\">".($i+1)."</a> "; }
	     if($prev_fl) { $prevID = $row[$field]; }
	     $i++;
        }
	   $str =  "<a href=\"$link&$field=$firstID\">$icon_first</a> <a href=\"$link&$field=$prevID\">$icon_prev</a> " . $str;
       $str .= "<a href=\"$link&$field=$nextID\">$icon_next</a  > <a href=\"$link&$field=$lastID\">$icon_last</a>";
	  }
     else
      {
       $pages = (int)($total/$max);
       $rest  = ($total/$max)-$pages;
       if($rest == 0) $pages--;
       if($pages > 0)
        {
         $prev_pg = $pg-1; if($prev_pg < 0)      $prev_pg = 0;
         $next_pg = $pg+1; if($next_pg > $pages) $next_pg = $pages;
         $str = "<a href=\"$link&pg=0\">$icon_first</a> <a href=\"$link&pg=$prev_pg\">$icon_prev</a> ";
         for($i = 0; $i <= $pages; $i++)
          {
           if($pg == $i) $str .= ($i+1)." ";
           else          $str .= "<a href=\"$link&pg=$i\">".($i+1)."</a> ";
          }
         $str .= "<a href=\"$link&pg=$next_pg\">$icon_next</a> <a href=\"$link&pg=$pages\">$icon_last</a> ";
        }
       else $str = '&nbsp;';
      }
     return $str;
    }

   public static function makeNotFoundItem($w3,$colspan = 0,$only_label = true,$no_table = false,$label = 1217)
    {
	 // Create a generic not found item
	 if($only_label) return $w3->getWord($label);

	 ob_start();
     require($w3->getSkinObj()->getListItemStart(0,false,false,false,false));
     $str = ob_get_contents();
	 ob_end_clean();

     ob_start();
     $colspan = "colspan=\"$colspan\"";
	 $content = $w3->getWord($label);
	 $width   = '';
	 require($w3->getSkinObj()->getListItem(0,false,false,false,false));
	 $str .= ob_get_contents();
     ob_end_clean();

	 ob_start();
     require($w3->getSkinObj()->getListItemEnd(0,false,false,false));
     $str .= ob_get_contents();
     ob_end_clean();

	 if($no_table) return $str;
     ob_start();
	 require($w3->getSkinObj()->getListBottomEnd());
	 $str .= ob_get_contents();
     ob_end_clean();

     return $str;
    }

   public function getPage() { return $this->page; }

   public function getMaxPageItems() { return $this->max_page_items; }

   public function getTotalItems() { return $this->total_items; }

   public function setPage($page) { $this->page = $page; }

   public function setMaxPageItems($max_page_items) { $this->max_page_items = $max_page_items; }

   public function setObjectsPerRow($objects_per_row) { $this->objects_per_row = $objects_per_row; }

   public function setPagesNavigatorLink($pages_navigator_link) { $this->pages_navigator_link = $pages_navigator_link; }

   public function setSearchForm($search_form) { $this->search_form = $search_form; }

   public function setCaptionLabels($labels) { $this->_CAPTION_LABELS = $labels; }
   
   public function setCaptionWidths($widths) { $this->_CAPTION_WIDTHS = $widths; }

   public function setCaptionColspan($colspan) { $this->_CAPTION_COLSPAN = $colspan; }

   public function setWidths($widths) { $this->_WIDTHS = $widths; }

   public function setCurrentID($currentID) { $this->currentID = $currentID; }

   public function setRenderCaption($render_caption) { $this->render_caption = $render_caption; }

   public function setRenderTable($render_table) { $this->render_table = $render_table; }

   public function setUseColors($use_colors) { $this->use_colors = $use_colors; }
   
   public function setRenderPagesNavigator($render_pages_navigator) { $this->render_pages_navigator = $render_pages_navigator; }
   
   public function setTotalItems($total_items) { $this->total_items = $total_items; }
   
   public function setBottomContent($bottom_content) { $this->bottom_content = $bottom_content; }

   public function setExtraHeader($extra_header,$i = 0) { $this->_EXTRA_HEADER[$i] = $extra_header; }
   
   public function setEmptyLabel($empty_label) { $this->empty_label = $empty_label; }

   public function setColumns($columns) { $this->columns = $columns; }

   public function setInvertColors($invert_colors) { $this->invert_colors = $invert_colors; }

   public function setPageNavigatorField($page_navigator_field) { $this->page_navigator_field = $page_navigator_field; }

   public function setPageNavigatorID($page_navigator_id) { $this->page_navigator_id = $page_navigator_id; }

   public function setPageNavigatorRS($page_navigator_rs) { $this->page_navigator_rs = $page_navigator_rs; }
   
   public function setUseAutoPage($use_auto_page) { $this->use_auto_page = $use_auto_page; }
   
   public function setSeparator($separator) { $this->_SEPARATOR = $separator; }

   public function setBarVertical($bar_vertical) { $this->_BAR_VERTICAL = $bar_vertical; }

   public function setBarHorizontal($bar_horizontal) { $this->_BAR_HORIZONTAL = $bar_horizontal; }

  }

?>
