<?php

 // THIS FILE IS PART OF WEBMATIC3 (Webmatic cube)
 // www.valarsoft.com

 defined('WM3_cEXEC') or die('Denied');

 class webmatic3
  {
   // Permanent objects
   private $_language,$_skin,$_panel,$_page,$_section,$_db;

   // Variables
   private $stage,$section,$action;
   private $authenticate_userID,$authenticate_adminID;
   private $home_languageID,$admin_languageID,$preferences_languageID;
   private $setupOn,$warningOn,$debugOn,$setup_ip;

   // Arrays
   private $_MENUS,$_PREFERENCES,$_PREFERENCESS_CONTENTS;

   function __construct($setup_on,$warning_on,$debug_on,$setup_ip)
    {
     $this->setupOn   = $setup_on;
     $this->warningOn = $warning_on;
     $this->debugOn   = $debug_on;
	 $this->setup_ip  = $setup_ip;

     // Parse some input
     wm3Util::stripMagicQuotes();
     wm3Util::setIDToInt();
     wm3Util::parseSearchFilter();

     // Autosetup
     if(!$this->setupOn) // Setup its disabled
      {
	   if(file_exists('wm3_config.php')) require_once('wm3_config.php');
	   else                              wm3Util::dieMessage("Setup error","Webmatic needs setup but setup is disabled. Please enable the setup!");
      }
     else
      {
       if(!wm3Setup::configNeeded($this,$this->setupOn)) // No setup needed
        {
         require_once('wm3_config.php');
        }
       else // Setup needed and run it
        {
		 if(!defined('WM3_cCONFIG_STARTING_LANGUAGE')) define('WM3_cCONFIG_STARTING_LANGUAGE',WM3_cLANGUAGE_EN); // Needed by language object when do the setup
		 $this->admin_languageID = wm3Language::writeCookieAdminLanguageID($this,0,true);
		 $this->_language = new wm3Language(null,$this->admin_languageID,true);
         wm3Setup::executeSetup($this,$this->setup_ip);
         // Security warning and debug informations in setup stage
         if($this->warningOn) $this->warning();
         if($this->debugOn)   $this->debug($this,true);	 
         exit; // Setup not complete, can not continue
        }
      }

     // Setup passed and webmatic can start
     $this->_db = new wm3DB(WM3_cCONFIG_DB_SERVER,WM3_cCONFIG_DB_USER,WM3_cCONFIG_DB_PASSWORD,WM3_cCONFIG_DB_NAME,WM3_cCONFIG_DB_PREFIX);
     $this->checkVariables();
     $this->_language              = new wm3Language($this);
     $this->home_languageID        = wm3Language::writeCookieHomeLanguageID($this);
     $this->admin_languageID       = wm3Language::writeCookieAdminLanguageID($this);
     $this->preferences_languageID = wm3Language::writeCookiePreferencesLanguageID($this); 
     $this->setPreferences($this);
     $this->setPreferencesContents($this);
     switch($this->getStage())
      {
       // Set correct stage language
       case WM3_cSTAGE_ADMIN: { $this->_language->getLanguageFile($this,$this->admin_languageID); break; }
       default:               { $this->_language->getLanguageFile($this,$this->home_languageID);  break; }
      }
     $this->_skin    = new wm3Skin($this,
	                               $this->_PREFERENCES['skin'],
                                   array(
                                         'logo'           => $this->_PREFERENCES['logo'],
                                         'logo_width'     => $this->_PREFERENCES['logo_width'],
                                         'logo_height'    => $this->_PREFERENCES['logo_height'],
                                         'logo_printable' => $this->_PREFERENCES['logo_printable'],
                                        ),
                                   array(
                                         $this->_PREFERENCES['banner1ID'],
                                         $this->_PREFERENCES['banner2ID'],
                                         $this->_PREFERENCES['banner3ID'],
                                         $this->_PREFERENCES['banner4ID'],
                                         $this->_PREFERENCES['banner5ID'],
                                         $this->_PREFERENCES['banner6ID'],
                                         $this->_PREFERENCES['banner7ID'],
                                         $this->_PREFERENCES['banner8ID'],
                                         $this->_PREFERENCES['banner9ID']
                                        )
                                  );
     $this->_panel   = new wm3Panel();
     $this->_page    = new wm3Page();
     $this->_section = new wm3Section();
     $this->authenticate_userID = $this->authenticate_adminID = WM3_cUSER_ANONYMOUS;
     $this->_MENUS   = wm3Menu::getMenusDataArray($this); // Take menus data from database     
    }

   public function exec()
    {
     $this->doAuthentication();
     wm3Session::manageSession($this);
     $this->createContent();
     $this->renderContent();
     $this->_db->close();
    }

   private function doAuthentication()
    {
     if($this->stage == WM3_cSTAGE_HOME)            $this->doAuthenticateHome();
     if($this->stage == WM3_cSTAGE_USER)            $this->doAuthenticateUser();
     if($this->stage == WM3_cSTAGE_ADMIN)           $this->doAuthenticateAdmin();
	 if($this->stage == WM3_cSTAGE_CHAT)            $this->doAuthenticateChat();
	 if($this->stage == WM3_cSTAGE_RSS)             $this->doAuthenticateRss();
	 if($this->stage == WM3_cSTAGE_BANNER_EXCHANGE) $this->doAuthenticateBannerExchange();
	 if($this->stage == WM3_cSTAGE_CAPTCHA)         $this->doAuthenticateCaptcha();
	 if($this->stage == WM3_cSTAGE_KERNEL)          $this->doAuthenticateKernel();
    }

   private function createContent()
    {
     $this->getSkinObj()->setCurrentSkinRender(WM3_cSKIN_RENDER_PAGE); // Set current skin render to page render
     if($this->stage == WM3_cSTAGE_HOME) {
     	                                  $this->_page->createHomePage($this);
     	                                  $this->_panel->createPanels($this);
     	                                 }
	 else                                {
                                          // Disable left/right panels for all not home stage
		                                  $this->_panel->setColumnEmpty(WM3_cPANEL_LEFT,true);
                                          $this->_panel->setColumnEmpty(WM3_cPANEL_RIGHT,true);
                                         }
     if($this->stage == WM3_cSTAGE_USER)            $this->_page->createUserPage($this);
     if($this->stage == WM3_cSTAGE_ADMIN)           $this->_page->createAdminPage($this);
	 if($this->stage == WM3_cSTAGE_CHAT)            $this->_page->createChatPage($this);
	 if($this->stage == WM3_cSTAGE_RSS)             $this->_page->createRssPage($this);
	 if($this->stage == WM3_cSTAGE_BANNER_EXCHANGE) $this->_page->createBannerExchangePage($this);
     if($this->stage == WM3_cSTAGE_CAPTCHA)         $this->_page->createCaptchaPage($this);     
     if($this->stage == WM3_cSTAGE_KERNEL)          $this->_page->createKernelPage($this);
    }

   private function renderContent()
    {
     // Adjust rows width
     $this->_skin->makeWidth(WM3_cPANEL_LEFT, $this);
     $this->_skin->makeWidth(WM3_cPANEL_RIGHT,$this);

     // Render head
     switch($this->stage)
      {
       case WM3_cSTAGE_ADMIN:           { require($this->_skin->getAdminHead($this)); break; }
       case WM3_cSTAGE_CHAT:            { require($this->_skin->getChatHead($this)); break; }
	   case WM3_cSTAGE_RSS:             { break; }
	   case WM3_cSTAGE_BANNER_EXCHANGE: { break; }
	   case WM3_cSTAGE_CAPTCHA:         { break; }
	   case WM3_cSTAGE_KERNEL:          { break; }
       default:                         { require($this->_skin->getHead($this)); }
      }

     // Render body
     switch($this->stage)
      {
	   case WM3_cSTAGE_RSS:             { break; }
	   case WM3_cSTAGE_BANNER_EXCHANGE: { break; }
	   case WM3_cSTAGE_CAPTCHA:         { break; }
	   case WM3_cSTAGE_KERNEL:          { break; }
       default:                         { require($this->_skin->getBody($this)); }
      }

     // Render foot
     switch($this->stage)
      {
       case WM3_cSTAGE_ADMIN:           { require($this->_skin->getAdminFoot($this)); break; }
	   case WM3_cSTAGE_CHAT:            { require($this->_skin->getChatFoot($this)); break; }
	   case WM3_cSTAGE_RSS:             { break; }
	   case WM3_cSTAGE_BANNER_EXCHANGE: { break; }
	   case WM3_cSTAGE_CAPTCHA:         { break; }
	   case WM3_cSTAGE_KERNEL:          { break; }
       default:                         { require($this->_skin->getFoot($this)); }
      }

     // Security warning and debug informations
	 if($this->stage != WM3_cSTAGE_RSS             &&
	    $this->stage != WM3_cSTAGE_BANNER_EXCHANGE &&
	    $this->stage != WM3_cSTAGE_CAPTCHA         &&
	    $this->stage != WM3_cSTAGE_KERNEL)
	  {
       if($this->setupOn)   $this->warningSetup();
       if($this->warningOn) $this->warning();
       if($this->debugOn)   $this->debug($this);
      }
    }

   private function doAuthenticateHome()
    {
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication($this);
     $this->authenticate_adminID = wm3User::checkAdminAuthentication($this);
    }

   private function doAuthenticateUser()
    {
     // TODO le variabili (in genere provenienti da campi delle form) vanno quotate? Si!, HQuot?
     // User authentication and logout
     if($this->action == WM3_cACTION_LOGOUT) wm3User::logoutUser();
     $this->authenticate_userID = wm3User::setUserCookies($this);
	 $this->authenticate_adminID = wm3User::checkAdminAuthentication($this);
    }

   private function doAuthenticateAdmin()
    {
     // TODO le variabili (in genere provenienti da campi delle form) vanno quotate? Si!, HQuot?
     // Admin authentication and logout
     if($this->action == WM3_cACTION_LOGOUT) wm3User::logoutAdmin();
     $this->authenticate_adminID = wm3User::setAdminCookies($this);
	 $this->authenticate_userID  = wm3User::checkUserAuthentication($this);
    }

   private function doAuthenticateChat()
    {
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication($this);
     $this->authenticate_adminID = wm3User::checkAdminAuthentication($this);
    }

   private function doAuthenticateRss()
    {//TODO serve per gli RSS?
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication($this);
     $this->authenticate_adminID = wm3User::checkAdminAuthentication($this);
    }

   private function doAuthenticateBannerExchange()
    {//TODO serve per gli RSS?
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication($this);
     $this->authenticate_adminID = wm3User::checkAdminAuthentication($this);
    }

   private function doAuthenticateCaptcha()
    {//TODO serve per gli RSS?
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication($this);
     $this->authenticate_adminID = wm3User::checkAdminAuthentication($this);
    }

   private function doAuthenticateKernel()
    {
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication($this);
     $this->authenticate_adminID = wm3User::checkAdminAuthentication($this);
    }

   private function checkVariables()
    {
     $this->stage = $this->section = $this->action = 0;
	 if(isset($_GET ['stage']))   $this->stage = abs(intval($_GET ['stage'])) > WM3_cSTAGE_KERNEL ? 0 : abs(intval($_GET ['stage'])); 
	 if(isset($_POST['stage']))   $this->stage = abs(intval($_POST['stage'])) > WM3_cSTAGE_KERNEL ? 0 : abs(intval($_POST['stage'])); 
     if(isset($_GET ['section'])) $this->section = intval($_GET ['section']);
     if(isset($_POST['section'])) $this->section = intval($_POST['section']);
     if(isset($_GET ['action']))  $this->action  = intval($_GET ['action']);
     if(isset($_POST['action']))  $this->action  = intval($_POST['action']);
    }

   public static function getPostValueArray($_FIELD,$quote_db = 0)
    {
     // Get all the post variables value needed and put it into the fields array
	 $_AUX = array();
     for($i = 0; $i < sizeof($_FIELD); $i++)
      {
       if($quote_db) $_AUX[$_FIELD[$i]] = wm3Util::quoteDB(webmatic3::getVariable($_FIELD[$i],'post'));
       else          $_AUX[$_FIELD[$i]] = webmatic3::getVariable($_FIELD[$i],'post');
      }
     return $_AUX;
    }

   public static function getRowValueArray($_FIELD,$row,$quote_db = 0)
    {
     // Get all $row values and put it into the fields array
     for($i = 0; $i < sizeof($_FIELD); $i++)
      {
       if(property_exists($row,$_FIELD[$i]))
        {
         if($quote_db) $_AUX[$_FIELD[$i]] = wm3Util::quoteDB($row->$_FIELD[$i]);
         else          $_AUX[$_FIELD[$i]] = $row->$_FIELD[$i];
	    }
      }
     return $_AUX;
    }

   public function includeJavaScript()
    {
     $js =  "<script language=\"JavaScript\" src=\"".$this->getSkinObj()->getJS()."\" type=\"text/javascript\"></script>\n".
            "  <script language=\"JavaScript\" src=\"".WM3_cPATH_JAVASCRIPT."wm3_constants.js\" type=\"text/javascript\"></script>\n".
	        "  <script language=\"JavaScript\" src=\"".WM3_cPATH_JAVASCRIPT."wm3_lib.js\" type=\"text/javascript\"></script>\n".
			"  <script language=\"JavaScript\" src=\"".WM3_cPATH_JAVASCRIPT."wm3_editor.js\" type=\"text/javascript\"></script>";
     if(wm3Social::isFacebookEnabled()) $js .= "\n  <script language=\"JavaScript\" src=\"".WM3_cPATH_JAVASCRIPT."wm3_social_facebook.js\" type=\"text/javascript\"></script>";
     if(wm3Social::isGPlusoneEnabled()) $js .= "\n  <script type=\"text/javascript\" src=\"http://apis.google.com/js/plusone.js\"></script>";
     if(wm3Social::isTwitterEnabled())  $js .= "\n  <script type=\"text/javascript\" src=\"http://platform.twitter.com/widgets.js\"></script>";
     
	 return $js;
    }

   public function setMenusArray($_ARRAY)      { return $this->_MENUS = $_ARRAY; }

   public function setPreferences($w3)         { $this->_PREFERENCES          = wm3Preferences::loadPreferences($w3); }
 
   public function setPreferencesContents($w3) { $this->_PREFERENCES_CONTENTS = wm3Preferences::loadPreferencesContents($w3,$this->home_languageID); }
 
   public function setHomeLanguage($languageID)  { $this->home_languageID = $languageID; }
   
   public function setAdminLanguage($languageID) { $this->admin_languageID = $languageID; }

   public function setPreferencesLanguage($languageID) { $this->preferences_languageID = $languageID; }
   
   public function setDBObj($obj) { $this->_db = $obj; }
 
   public function getPreferences($n)         { return $this->_PREFERENCES[$n]; }
   
   public function getPreferencesContents($n) { return $this->_PREFERENCES_CONTENTS[$n]; }
   
   public function getWord($i)                         { return $this->_language->getWord($i); }
   
   public function getWordHelp($i)                     { return $this->_language->getWordHelp($i); }

   public function getWordExpansion($i)                { return $this->_language->getWordExpansion($i); }

   public function getWordMonth($i)                    { $array = $this->_language->getArrayMonth(); return $array[$i-1]; }
   
   public function getWordDayOfTheWeek($i)             { $array = $this->_language->getArrayDayOfTheWeek(); return $array[$i-1]; }
   
   public function getWordZodiacSign($i,$info = false) { $array = $this->_language->getArrayZodiacSign(); if($info) return $array[$i-1][1]; else return $array[$i-1][0]; }

   public function getHomeLanguage() { return $this->home_languageID; }
   
   public function getAdminLanguage() { return $this->admin_languageID; }

   public function getPreferencesLanguage() { return $this->preferences_languageID; }
   
   public function getAuthenticateUserID() { return $this->authenticate_userID; }
   
   public function getAuthenticateAdminID() { return $this->authenticate_adminID; }
   
   public function getMenusArray() { return $this->_MENUS; }

   public function getAuthenticateAdminUserID()
    {
	 // Return authenticated adminID (or userID if administrator its not logged)
	 if($this->authenticate_adminID) return $this->authenticate_adminID;
	 return $this->authenticate_userID;
	}

   public function getDBObj() { return $this->_db; }

   public function getSkinObj() { return $this->_skin; }
   
   public function getPanelObj() { return $this->_panel; }
   
   public function getPageObj() { return $this->_page; }
   
   public function getLanguageObj() { return $this->_language; }

   public function getSectionObj() { return $this->_section; }

   public function getStage() { return $this->stage; }
   
   public function getSection() { return $this->section; }
   
   public function getAction() { return $this->action; }
   
   public function getVariable($name,$method = "both")
    {
     if($method == "get")  if(isset($_GET [$name])) return $_GET [$name];
     if($method == "post") if(isset($_POST[$name])) return $_POST[$name];
     if($method == "both")
      {
       if(isset($_GET [$name])) return $_GET [$name];
       if(isset($_POST[$name])) return $_POST[$name];
      }
   	}

   private function warningSetup()
    {
     if($this->setupOn)
      {
       $str = "<br><table width=\"50%\" border=\"1\" align=\"center\"><tr><td bgcolor=\"#ff2222\"><b>Warning:</b></td></tr>
                <tr><td bgcolor=\"#ffffff\">Setup its on.<br>Its very insicure keep it on, please set it to off from <b>index.php</b>.</td></tr>
               </table>";
       echo $str;
      }
    }

   private function warning()
    {
     if($this->warningOn)
      {
	   // TODO
	   if(!extension_loaded('imap') ||
	      !extension_loaded('gd')   ||
	      (!wm3Util::isFolderWritable(WM3_cPATH_IMAGES)     ||
	      !wm3Util::isFolderWritable(WM3_cPATH_ECOMMERCE)   ||
		  !wm3Util::isFolderWritable(WM3_cPATH_MEDIA_ALBUM) ||
		  !wm3Util::isFolderWritable(WM3_cPATH_DOWNLOADS)))
	    {
         $str  = "<br><table width=\"50%\" border=\"1\" align=\"center\"><tr><td colspan=\"3\" bgcolor=\"#ffaa33\"><b>Warning:</b></td></tr>";
         if(!extension_loaded('imap')) $str .= "<tr><td bgcolor=\"#ffffff\">IMAP extension</td><td bgcolor=\"#ffffff\">Not loaded</td><td bgcolor=\"#ffffff\">Must be loaded</td></tr>";
         if(!extension_loaded('gd'))   $str .= "<tr><td bgcolor=\"#ffffff\">GD extension</td><td bgcolor=\"#ffffff\">Not loaded</td><td bgcolor=\"#ffffff\">Must be loaded</td></tr>";
	     if(!wm3Util::isFolderWritable(WM3_cPATH_IMAGES)      ||
	        !wm3Util::isFolderWritable(WM3_cPATH_ECOMMERCE)   ||
		    !wm3Util::isFolderWritable(WM3_cPATH_MEDIA_ALBUM) ||
		    !wm3Util::isFolderWritable(WM3_cPATH_DOWNLOADS)) $str .= "<tr><td bgcolor=\"#ffffff\">Upload folders</td><td bgcolor=\"#ffffff\">Write permission not granted</td><td bgcolor=\"#ffffff\">Write permissions must be enabled on folders</td></tr>";
         $str .= "</table>";
         echo $str;
	    }
      }
    }
   
   private function debug($w3,$setup = false)
    {
     global $WM3_cCLASSES_LOADED;

     $str = "<br><table width=\"50%\" border=\"1\" align=\"center\"><tr bgcolor=\"#00ee00\"><td colspan=\"5\"><b>Debug information:</b></td></tr>";
     $col="#ffffff";

     // Post variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Post variables (_POST):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_POST,true)."</pre></td>
              </tr>";

     // Get variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Get variables (_GET):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_GET,true)."</pre></td>
              </tr>";

     // Cookies variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Cookies variables (_COOKIE):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_COOKIE,true)."</pre></td>
              </tr>";

     // Session variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Session variables (_SESSION):</b></td></tr>";
	 if(isset($_SESSION))
      $str .= "<tr bgcolor=\"$col\">
                <td colspan=\"2\"><pre>".print_r($_SESSION,true)."</pre></td>
               </tr>";
	 else $str .= "<tr bgcolor=\"$col\"><td colspan=\"2\"><pre>".print_r(array(),true)."</pre></td></tr>";

     // Uploaded files
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Uploaded files (_FILES):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_FILES,true)."</pre></td>
              </tr>";

     // Server variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Server variables (_SERVER):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_SERVER,true)."</pre></td>
              </tr>";              

     // Classes loaded
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Classes loaded:</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\">$WM3_cCLASSES_LOADED</td>
              </tr>"; 

     // Other informations
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Other informations:</b></td></tr>";
     // Check if setup is running because some debug information use the database during setup
	 if(!$setup) {
		          $str .= "<tr bgcolor=\"#ffffff\"><td><b>Stage (filtered value)</b></td> <td><b>".$this->getStage()."</b></td></tr>";
	              $str .= "<tr bgcolor=\"#dddddd\"><td><b>Current session ID</b></td><td><b>".wm3Session::getSessionID($this)."</b></td></tr>";
	              $str .= "<tr bgcolor=\"#ffffff\"><td><b>WM3_cQUERY_AFFECTED</b></td> <td><b>".$w3->getDBObj()->getQueriesAffected()."</b></td></tr>";
			     }
	 else        {
		          $str .= "<tr bgcolor=\"#ffffff\"><td><b>Stage (filtered value)</b></td> <td><b>-</b></td></tr>";
	              $str .= "<tr bgcolor=\"#dddddd\"><td><b>Current session ID</b></td><td><b>-</b></td></tr>";
	              $str .= "<tr bgcolor=\"#ffffff\"><td><b>WM3_cQUERY_AFFECTED</b></td> <td><b>-</b></td></tr>";
			     }
     $str .= "<tr bgcolor=\"#dddddd\"><td><b>Memory usage</b></td><td><b>".memory_get_usage()." bytes</b></td></tr>";
     $str .= "<tr bgcolor=\"#ffffff\"><td><b>Page generated time</b></td><td><b>".wm3Util::delayTime()."</b></td></tr>";
     $str .= "</table><br>";
     echo $str;
    }

  }

?>
